@@ -1,4 +1,4 @@
-from .base import HTMLGenerator, yield_children
+from .base import HTMLGenerator, yield_child
 
 class Join(HTMLGenerator):
     '''
@@ -11,8 +11,15 @@ class Join(HTMLGenerator):
     def __iter__(self):
         first_item = True
         for child in self.iterable :
+            # Skip "explicitly empty" items, just like all other generators
+            # Be sure we do NOT add a joiner for this empty item
+            if child is None or child is False :
+                return
+
             if not first_item :
                 yield from self.joiner
             else :
                 first_item = False
-            yield from child
\ No newline at end of file
+
+            # child could be another HTMLGenerator, iterable, etc.
+            yield from yield_child(child)
\ No newline at end of file
